/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;

/**
 * Main application class
 * @author akuehntopf
 */
public class StreamtasticApp 
{
    /**
     * Returns the application's Version
     * @return version
     */
    public static String 
    getVersion()
    {
        return "0.4.1";
    }
    
    /**
     * Program entry point
     * @param args the commandline arguments
     */
    public static void 
    main(String[] args)
    throws UnsupportedLookAndFeelException
    {
        // install Substance LAF
        try
        {
            // new default tabbed pane border in substance does not look nice
            // full revert to old look is unfortunately not possible
            // see About Dialog for effects of this property
            Object TABBED_PANE_CONTENT_BORDER_KIND = Class.forName(
                    "org.pushingpixels.substance.api.SubstanceLookAndFeel").getDeclaredField(
                    "TABBED_PANE_CONTENT_BORDER_KIND").get(null);
            Object SINGLE_FULL = Class.forName(
                    "org.pushingpixels.substance.api.SubstanceConstants$TabContentPaneBorderKind")
                    .getDeclaredField("SINGLE_FULL").get(null);
            UIManager.put(TABBED_PANE_CONTENT_BORDER_KIND, SINGLE_FULL);

            // install LAFs
            String[][] substanceLAFs = new String[][] {
                    { "Substance (Autumn)", "org.pushingpixels.substance.api.skin.SubstanceAutumnLookAndFeel" },
                    //{ "Substance (BusinessBlackSteel)", "org.pushingpixels.substance.api.skin.SubstanceBusinessBlackSteelLookAndFeel" },
                    { "Substance (BusinessBlueSteel)", "org.pushingpixels.substance.api.skin.SubstanceBusinessBlueSteelLookAndFeel" },
                    { "Substance (Business)", "org.pushingpixels.substance.api.skin.SubstanceBusinessLookAndFeel" },
                    //{ "Substance (ChallengerDeep)", "org.pushingpixels.substance.api.skin.SubstanceChallengerDeepLookAndFeel" },
                    { "Substance (CremeCoffee)", "org.pushingpixels.substance.api.skin.SubstanceCremeCoffeeLookAndFeel" },
                    { "Substance (Creme)", "org.pushingpixels.substance.api.skin.SubstanceCremeLookAndFeel" },
                    //{ "Substance (DustCoffee)", "org.pushingpixels.substance.api.skin.SubstanceDustCoffeeLookAndFeel" },
                    { "Substance (Dust)", "org.pushingpixels.substance.api.skin.SubstanceDustLookAndFeel" },
                    { "Substance (EmeraldDusk)", "org.pushingpixels.substance.api.skin.SubstanceEmeraldDuskLookAndFeel" },
                    //{ "Substance (Gemini)", "org.pushingpixels.substance.api.skin.SubstanceGeminiLookAndFeel" },
                    //{ "Substance (GraphiteAqua)", "org.pushingpixels.substance.api.skin.SubstanceGraphiteAquaLookAndFeel" },
                    { "Substance (GraphiteGlass)", "org.pushingpixels.substance.api.skin.SubstanceGraphiteGlassLookAndFeel" },
                    { "Substance (Graphite)", "org.pushingpixels.substance.api.skin.SubstanceGraphiteLookAndFeel" },
                    //{ "Substance (Magellan)", "org.pushingpixels.substance.api.skin.SubstanceMagellanLookAndFeel" },
                    //{ "Substance (Mariner)", "org.pushingpixels.substance.api.skin.SubstanceMarinerLookAndFeel" },
                    { "Substance (MistAqua)", "org.pushingpixels.substance.api.skin.SubstanceMistAquaLookAndFeel" },
                    //{ "Substance (MistSilver)", "org.pushingpixels.substance.api.skin.SubstanceMistSilverLookAndFeel" },
                    //{ "Substance (Moderate)", "org.pushingpixels.substance.api.skin.SubstanceModerateLookAndFeel" },
                    //{ "Substance (NebulaBrickWall)", "org.pushingpixels.substance.api.skin.SubstanceNebulaBrickWallLookAndFeel" },
                    { "Substance (Nebula)", "org.pushingpixels.substance.api.skin.SubstanceNebulaLookAndFeel" },
                    //{ "Substance (OfficeBlack2007)", "org.pushingpixels.substance.api.skin.SubstanceOfficeBlack2007LookAndFeel" },
                    { "Substance (OfficeBlue2007)", "org.pushingpixels.substance.api.skin.SubstanceOfficeBlue2007LookAndFeel" },
                    { "Substance (OfficeSilver2007)", "org.pushingpixels.substance.api.skin.SubstanceOfficeSilver2007LookAndFeel" },
                    { "Substance (Raven)", "org.pushingpixels.substance.api.skin.SubstanceRavenLookAndFeel" },
                    //{ "Substance (Sahara)", "org.pushingpixels.substance.api.skin.SubstanceSaharaLookAndFeel" },
                    { "Substance (Twilight)", "org.pushingpixels.substance.api.skin.SubstanceTwilightLookAndFeel" }
            };
            for (String[] substanceLAF: substanceLAFs) 
                UIManager.installLookAndFeel(substanceLAF[0], substanceLAF[1]);
        }
        catch (Exception e)
        {
            System.err.println("Ensure substance.jar is located in ./libs directory! ");
        }

        // since substance 5.3 LAF must be set on event dispatching thread
        SwingUtilities.invokeLater(new Runnable() { public void run() {
            // Set LAF
            try
            {
                UIManager.setLookAndFeel(Config.getInstance().getLookAndFeelClassName());
            }
            catch (Exception e)
            {
                System.err.println("Error setting look and feel ("
                        + Config.getInstance().getLookAndFeelClassName() + ")\n" + "\tcause: "
                        + e.getMessage());
                try
                {
                    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                    Config.getInstance().setProperty(Config.Key.LOOK_AND_FEEL, "Java");
                    System.err.println("Falling back to cross platform look and feel!");
                }
                catch (Exception ex) {}
            }
            boolean defaultLookAndFeelDecorated = UIManager.getLookAndFeel().getSupportsWindowDecorations();
            JFrame.setDefaultLookAndFeelDecorated(defaultLookAndFeelDecorated);
            JDialog.setDefaultLookAndFeelDecorated(defaultLookAndFeelDecorated);        
        
            // Start Frame
            new StreamtasticFrame().setVisible(true);
        }});
    }
}
