/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.icecastfeed;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.core.config.Config.Key;

/**
 * Represents an Icecast Station
 * 
 * @author mikeL@launchpad.net
 */
public class IcecastStation extends Station {

    private static final String PROPERTY_NAME = "server_name";
    private final static String PROPERTY_URL = "listen_url";
    private final static String PROPERTY_MIMETYPE = "server_type";
    private final static String PROPERTY_BITRATE = "bitrate";
    private final static String PROPERTY_GENRE = "genre";
    private final static String PROPERTY_CURRENT_TRACK = "current_song";
    
    private static Set <String> filterList = new HashSet<String>();
    static 
    { 
        String filter = Config.getInstance().getProperty(Key.ICECAST_GENRE_BLACKLIST);
        if (filter != null)
            filterList.addAll(Arrays.asList(filter.split("\\s")));

        //react to config changes
        Config.getInstance().addPropertyChangeListener(Key.ICECAST_GENRE_BLACKLIST, new PropertyChangeListener() {
            
            @Override
            public void propertyChange(PropertyChangeEvent evt)
            {
                filterList.clear();
                String newValue = (String)evt.getNewValue();
                if (newValue == null) return;
                filterList.addAll(Arrays.asList(newValue.split("\\s")));
            }
        });
    }
    
    private Set<String> genres = new TreeSet<String>();


    public IcecastStation(Properties attributes)
    {
        setName(attributes.getProperty(PROPERTY_NAME));
        setUrl(attributes.getProperty(PROPERTY_URL));
        setMimetype(attributes.getProperty(PROPERTY_MIMETYPE));
        try
        {
            int bitrate = Integer.parseInt(attributes.getProperty(PROPERTY_BITRATE));
            // if bitrate is greater than 10000, consider unit as bits and transform to kbits
            setBitrate(bitrate > 10000 ? bitrate / 1000 : bitrate);
        } catch (NumberFormatException Ex)
        {
            setBitrate(null);
        }
        setGenre(attributes.getProperty(PROPERTY_GENRE));
        setCurrentTrack(attributes.getProperty(PROPERTY_CURRENT_TRACK));

    }

    protected int removeGenre(String genre) 
    {
        genres.remove(genre);

        return genres.size();
    }
    

    public String[] getGenres()
    {
        return genres.toArray(new String[genres.size()]);
    }

    @Override
    public void setGenre(String genreString) {
        super.setGenre(genreString);

        this.genres.clear();
        String[] genres = genreString.split("\\s");

        for (String genre : genres)
            if (genre.length() > 2 && !filterList.contains(genre))
                this.genres.add(genre);
    }

}
