/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.core.shoutcastfeed;

import org.kuehntopf.streamtastic.core.Station;

/**
 * Represents a Shoutcast Station
 * @author akuehntopf
 */
public class ShoutcastStation extends Station
{
    private Integer  _id;
    
    /**
     * Returns this Station's bitrate
     * @return bitrate
     */
    public Integer
    getId() 
    {
        return _id;
    }

    /**
     * Sets this Station's id
     * @param id
     */
    public void 
    setId(Integer id) 
    {
        this._id = id;
        setUrl("http://yp.shoutcast.com/sbin/tunein-station.pls?id=" + id);
    }

    /**
     * Compares this ShoutcastStation to other 
     * ShoutcastStations
     * @param obj Object to compare with
     * @return are those Objects equal?
     */
    @Override
    public boolean 
    equals(Object obj) 
    {
        if (obj instanceof ShoutcastStation)
        {
            ShoutcastStation s = (ShoutcastStation)obj;
            return s.getId().equals(this.getId());
        } else if (obj instanceof Station)
        {
            Station s = (Station)obj;
            return s.getUrl().equals(this.getUrl());
        } else
        {
            return super.equals(obj);
        }
    }

    /**
     * Returns the HashCode for this ShoutcastStation
     * This is required for correct equals behavior.
     * @return hashcode of this Object
     */
    @Override
    public int 
    hashCode() 
    {
        int hash = 7;
        hash = 59 * hash + (this._id != null ? this._id.hashCode() : 0);
        return hash;
    }
}
