/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui;

import java.awt.Font;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFrame;

import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class StreamtasticTrayIcon extends TrayIcon implements WindowStateListener, ActionListener {

    private PopupMenu menu;
    private boolean visible = false;
    private StreamtasticFrame frame;
    private int frameState = 0;

    public StreamtasticTrayIcon(StreamtasticFrame frame)
    {
        super(frame.getIconImage(), frame.getTitle());
        this.frame = frame;
        setImageAutoSize(true);

        frame.addWindowStateListener(this);
        addActionListener(this);

        menu = new PopupMenu("Streamtastic");
        MenuItem restoreItem = new ActionMenuItem(new AbstractAction(Utilities
                .getString("MAINFRAME_RESTORE_WINDOW")) {
            public void actionPerformed(ActionEvent e)
            {
                StreamtasticTrayIcon.this.actionPerformed(e);
            }
        });
        menu.add(restoreItem);
        menu.add(new ActionMenuItem(frame.getSharedAction(StreamtasticFrame.GUIAction.ABOUT)));
        menu.addSeparator();
        menu.add(new ActionMenuItem(frame.getSharedAction(StreamtasticFrame.GUIAction.CLOSE)));
        setPopupMenu(menu);

        // set bold font for default action
        menu.setFont(frame.getFont());
        restoreItem.setFont(menu.getFont().deriveFont(Font.BOLD));
    }
    
    public void setVisible(boolean visible)
    {
        if (!SystemTray.isSupported())
            throw new UnsupportedOperationException("Plattform does not support tray icons!");

        if (visible && !this.visible)
            try
            {
                SystemTray.getSystemTray().add(this);
                this.visible = true;
            }
            catch (Exception awte)
            {
                // should never happen because we already check SystemTray.isSupported()
                awte.printStackTrace();
            }
        else if (this.visible)
        {
            SystemTray.getSystemTray().remove(this);
            if (!frame.isVisible()) frame.setVisible(true);
            this.visible = false;
        }
    }

    public void dispose()
    {
        frame.removeWindowStateListener(this);
        setVisible(false);
    }

    public void windowStateChanged(WindowEvent e)
    {
        if ((e.getNewState()&JFrame.ICONIFIED)!=0) {
            frameState = e.getOldState();
            setVisible(true);
            frame.setVisible(!visible);
        }
    }

    public void actionPerformed(ActionEvent e)
    {
        if (!frame.isVisible()) 
        {
            frame.setVisible(true);
            frame.setExtendedState(frameState);
            setVisible(false);
        }
    }
}

class ActionMenuItem extends MenuItem implements PropertyChangeListener {

    protected ActionMenuItem(Action action)
    {
        super(action.getValue(Action.NAME).toString());
        addActionListener(action);
        action.addPropertyChangeListener(this);
        
        initActionProperties(action);
    }

    private void initActionProperties(Action action)
    {
        setLabel(action.getValue(Action.NAME).toString());
        setEnabled(action.isEnabled());
        setActionCommand((String) action.getValue(Action.ACTION_COMMAND_KEY));
        
        Integer key = (Integer) action.getValue(Action.MNEMONIC_KEY);
        setShortcut(new MenuShortcut(key==null?'\0':key));
    }

    public void propertyChange(PropertyChangeEvent evt)
    {
        if ("enabled".equals(evt.getPropertyName()))
        {
            setEnabled((Boolean) evt.getNewValue());
        }

        // TODO update item state according to action's state
        // currently only "enabled" is supported

        // print message + stacktrace for non handled properties
        else 
            new Exception("not handled: "+evt.getPropertyName()+": "+evt.getOldValue()+" -> "+evt.getNewValue()).printStackTrace();
    }

}
