/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.components;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.kuehntopf.streamtastic.core.Station.Property;

/**
 * @author Michael Lippold <thabird@free.fr>
 *
 */
public class ColumnSelectionPopupMenu extends JPopupMenu implements ItemListener, PopupMenuListener {

    private TableColumnModel model;
    private boolean ignoreNextInvokation = false;

    public ColumnSelectionPopupMenu(TableColumnModel model) {
        this.model = model;
        this.addPopupMenuListener(this);
        for (int i=0; i < model.getColumnCount(); i++)
        {
            TableColumn col = model.getColumn(i);
            JCheckBoxMenuItem mi = new JCheckBoxMenuItem(col.getHeaderValue().toString(), true);
            mi.putClientProperty(ColumnSelectionPopupMenu.class, col);
            mi.addItemListener(this);
            add(mi);
        }
    }

    public void setSelected(Property property, boolean b)
    {   
        StationColumn col = (StationColumn)model.getColumn(model.getColumnIndex(property));
        Object[] comps =  getComponents();
        for(Object co : comps) {
            if (!(co instanceof JCheckBoxMenuItem))continue; 
            JCheckBoxMenuItem mi = (JCheckBoxMenuItem)co;
                if (mi.getClientProperty(ColumnSelectionPopupMenu.class)==col)
                    mi.setState(b);              
        }
     }

    // Itemlistener
    @Override
    public void itemStateChanged(ItemEvent e)
    {
        if (ignoreNextInvokation) 
        {
            ignoreNextInvokation = false;
            return;
        }

        JCheckBoxMenuItem mi = (JCheckBoxMenuItem)(e.getSource());
        final StationColumn column = (StationColumn) mi.getClientProperty(ColumnSelectionPopupMenu.class);

        if (model.getColumnCount()==1 && !mi.isSelected())
        {
            ignoreNextInvokation = true;
            mi.setState(true);
            return;
        }

        if (mi.isSelected()) model.addColumn(column);
        else model.removeColumn(column);
    }

    // PopupMenuListener
    @Override
    public void popupMenuWillBecomeVisible(PopupMenuEvent e) 
    {
        if (!(getInvoker() instanceof JTableHeader)) return;

        // If a popup is set for the table header using table.getTableHeader.setComponentPopupmenu()
        // the mouse release event will be consumed by the popup menu invocation.
        // This results in a non finished column drag operation that causes repaint problems and will
        // throw an exception in case that the dragged column is removed.
        // To avoid this buggy behaviour the dragstate is reseted manually by calling JTableHeader.setDraggedColumn(null). 
        ((JTableHeader)getInvoker()).setDraggedColumn(null);
        getInvoker().repaint();
    }

    @Override public void popupMenuCanceled(PopupMenuEvent e) {}
    @Override public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {}
}
