/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.components;

import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.Action;
import javax.swing.JTable;
import javax.swing.RowSorter;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.gui.models.BitrateListModel;
import org.kuehntopf.streamtastic.gui.models.GenreListModel;
import org.kuehntopf.streamtastic.gui.models.StationTableModel;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class DirectoryPanel extends javax.swing.JPanel {

    /** Creates new form DirectoryPanel */
    public DirectoryPanel() {
        initComponents();
    }

    public void initGenreList(GenreListModel genreModel, ListSelectionListener listener)
    {
        genreList.setModel(genreModel);
        genreList.addListSelectionListener(listener);
    }

    public JTable initTable(TableModel model, TableColumnModel columnModel, RowSorter rowsorter)
    {
        stationTable.setAutoCreateColumnsFromModel(false);
        stationTable.setModel(model);
        stationTable.setColumnModel(columnModel);
        stationTable.setRowSorter(rowsorter);

        return stationTable;
    }

    public void initBitrates(BitrateListModel model, ActionListener listener)
    {
        bitrateComboBox.setModel(model);
        bitrateComboBox.addActionListener(listener);
    }

    public void setSearchAction(Action searchAction)
    {
        searchTextfield.addActionListener(searchAction);
    }

    public void setGenres(List<String> genres)
    {
        ((GenreListModel) genreList.getModel()).setGenres(genres, genreList.getSelectedIndex());
    }

    public void setStations(List<Station> stations)
    {
        // reset bitrate model
        ((BitrateListModel) bitrateComboBox.getModel()).reset();

        int currentRow = stationTable.getSelectedRow();
        ((StationTableModel) stationTable.getModel()).setStations(stations, currentRow == -1 ? -1
                : stationTable.convertRowIndexToModel(currentRow));

        // scroll viewport to selected row after update
        currentRow = stationTable.getSelectedRow();
        currentRow = currentRow == -1 ? 0 : currentRow;
        stationTable.scrollRectToVisible(stationTable.getCellRect(currentRow, 0, true));
    }

    public Station getSelectedStation()
    {
        int selPos = stationTable.getSelectedRow();
        if (selPos == -1)
            return null;

        return (Station) stationTable.getValueAt(selPos, 0);
    }

    public Integer getSelectedBitrate()
    {
        return (bitrateComboBox.getSelectedIndex() != 0) ? (Integer) bitrateComboBox
                .getSelectedItem() : null;
    }

    public String getSelectedGenre()
    {
        return (String) genreList.getSelectedValue();
    }

    public String getSearchTerm()
    {
        return searchTextfield.getText();
    }

    public void clearGenreSelection()
    {
        genreList.clearSelection();
    }

    public void clearStations()
    {
        ((StationTableModel) stationTable.getModel()).clear();
    }

    public void clearSearchTerm()
    {
        searchTextfield.setText("");
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        searchLabel = new javax.swing.JLabel();
        searchTextfield = new javax.swing.JTextField();
        bitrateLabel = new javax.swing.JLabel();
        bitrateComboBox = new javax.swing.JComboBox();
        jSplitPane1 = new javax.swing.JSplitPane();
        listScrollPane = new javax.swing.JScrollPane();
        genreList = new javax.swing.JList();
        tableScrollPane = new javax.swing.JScrollPane();
        stationTable = new javax.swing.JTable();

        setLayout(new java.awt.GridBagLayout());

        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("org/kuehntopf/streamtastic/resources/i18n/streamtastic"); // NOI18N
        searchLabel.setText(bundle.getString("DIR_SEARCH_LBL")); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 2, 4, 5);
        add(searchLabel, gridBagConstraints);

        searchTextfield.setToolTipText(bundle.getString("DIR_SEARCH_TT")); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 4, 0);
        add(searchTextfield, gridBagConstraints);

        bitrateLabel.setText(bundle.getString("DIR_SEARCH_BR")); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 5, 4, 5);
        add(bitrateLabel, gridBagConstraints);

        bitrateComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        bitrateComboBox.setPrototypeDisplayValue("xxxxx");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(4, 0, 4, 0);
        add(bitrateComboBox, gridBagConstraints);

        jSplitPane1.setDividerLocation(150);
        jSplitPane1.setDividerSize(4);

        genreList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        listScrollPane.setViewportView(genreList);

        jSplitPane1.setLeftComponent(listScrollPane);

        stationTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        tableScrollPane.setViewportView(stationTable);

        jSplitPane1.setRightComponent(tableScrollPane);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jSplitPane1, gridBagConstraints);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox bitrateComboBox;
    private javax.swing.JLabel bitrateLabel;
    private javax.swing.JList genreList;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JScrollPane listScrollPane;
    private javax.swing.JLabel searchLabel;
    private javax.swing.JTextField searchTextfield;
    private javax.swing.JTable stationTable;
    private javax.swing.JScrollPane tableScrollPane;
    // End of variables declaration//GEN-END:variables

}
