/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.components;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import org.kuehntopf.streamtastic.gui.views.View;

/**
 * ListCellRenderer for use in Sidebar Menu.
 * Renders JLabels
 * @author akuehntopf
 */
public class SidebarMenuCellRenderer implements ListCellRenderer 
{
    private ListCellRenderer LISTCELLRENDERER = new JList().getCellRenderer();

    /**
     * @param parent
     */
    public SidebarMenuCellRenderer(final JList parent)
    {
        super();
        // replace cell renderer template on LAF change
        parent.addPropertyChangeListener("UI", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt)
            {
                LISTCELLRENDERER = new JList().getCellRenderer();
            }
        });
    }

    /**
     * Returns the CellRendererComponent for a specific Object
     */
    public Component 
    getListCellRendererComponent(JList list, Object value,
                        int index, boolean isSelected, boolean cellHasFocus) 
    {
        View view = (View) value;
        Component c = LISTCELLRENDERER.getListCellRendererComponent(list, view.getDisplayName(), index, isSelected, cellHasFocus);

        if (c instanceof JLabel) 
        {
            JLabel label = (JLabel) c; 
            label.setIcon(view.getDisplayIcon());
        }

        return c;
    }
}
