/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.models;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.AbstractListModel;
import javax.swing.ComboBoxModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class BitrateListModel extends AbstractListModel implements TableModelListener,
        ComboBoxModel {

    private Set<Integer> bitrates = new TreeSet<Integer>(BASIC_SET);

    private static final String ALL = Utilities.getString("DIR_SEARCH_BR_ALL");
    private static final List<Integer> BASIC_SET = Arrays.asList(new Integer[] { 32, 64, 96, 128,
            160, 192, 256, 320 });

    private Integer[] cache = null;

    private Integer selected = null;

    /**
     * 
     */
    public BitrateListModel(StationTableModel bitrateProvider)
    {
        bitrateProvider.addTableModelListener(this);
    }

    public Object getSelectedItem()
    {
        if (selected == null || !bitrates.contains(selected))
            return getElementAt(0);
        return selected;
    }

    public void setSelectedItem(Object anItem)
    {
        if (anItem instanceof Integer && bitrates.contains(anItem))
        {
            selected = (Integer) anItem;
            return;
        }

        selected = null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.ListModel#getElementAt(int)
     */
    public Object getElementAt(int index)
    {
        if (index == 0)
            return ALL;
        return get(index - 1);
    }

    private Object get(int i)
    {
        if (cache == null)
        {
            cache = bitrates.toArray(new Integer[bitrates.size()]);
        }
        return cache[i];
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.ListModel#getSize()
     */
    public int getSize()
    {
        return bitrates.size() + 1;
    }

    public void tableChanged(TableModelEvent e)
    {
        if (e.getType() == TableModelEvent.DELETE)
            return;

        TableModel tm = (TableModel) e.getSource();
        int oldSize = bitrates.size();

        if (e.getLastRow() < tm.getRowCount()) // avoid ..OutOfBoundsException on fireTableDataChanged()
            for (int i = e.getFirstRow(); i <= e.getLastRow(); i++)
            {
                Integer bitrate = ((Station) tm.getValueAt(i, 0)).getBitrate();
                if (bitrate != null)
                    bitrates.add(bitrate);
            }

        if (bitrates.size() == oldSize)
            return;

        cache = null;
        fireContentsChanged(this, 0, bitrates.size());
    }

    public void reset()
    {
        if (selected!=null)
            return;

        // switch back to basic set, only if ALL was selected
        bitrates.clear();
        bitrates.addAll(BASIC_SET);
        cache=null;
        fireContentsChanged(this, 0, bitrates.size());
    }

}
