/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.kuehntopf.streamtastic.gui.models;

import java.util.Arrays;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.kuehntopf.streamtastic.core.CompoundMessage;
import org.kuehntopf.streamtastic.core.RippingManager;
import org.kuehntopf.streamtastic.core.Station;

public class DownloadsTableModel extends AbstractTableModel implements Observer
{

    private List<Station> _stations;

    /**
     * Constructs a new Downloads Table Model
     */
    public DownloadsTableModel() {
        _stations = new Vector<Station>(Arrays.asList(RippingManager.getInstance().getRippedStations()));
        RippingManager.getInstance().addObserver(this);
    }

    /**
     * Returns the number of rows
     * @return the number of rows
     */
    public int getRowCount() {
        return _stations.size();
    }

    /**
     * Returns the number of columns
     * @return the number of columns
     */
    public int getColumnCount() {
        return Station.Property.values().length;
    }

    /**
     * Returns a Value from the model
     * for a specific row and column
     * @param row row
     * @param col column
     * @return the value for the specified row and column
     */
    public Object getValueAt(int row, int col) {
        return _stations.get(row);
    }

    /**
     * Observer method which gets called
     * whenever the RippingManager changes its status
     * @param arg0
     * @param arg1
     */
    public void update(Observable obs, Object obj) {
        CompoundMessage msg = (CompoundMessage) obj;
        Station s = (Station) msg.getObject();
        int pos = _stations.indexOf(s);
        switch (msg.getMessage()) {
            case RIP_UPDATED:
                fireTableRowsUpdated(pos, pos);
                break;

            case RIP_REMOVED:
                _stations.remove(s);
                fireTableRowsDeleted(pos, pos);
                break;

            case RIP_CREATED:
                _stations.add(s);
                pos = _stations.indexOf(s);
                fireTableRowsInserted(pos, pos);
                break;

            default:
                fireTableDataChanged();
        }
    }
}
