/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kühntopf <andreas@kuehntopf.org>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.models;

import java.util.List;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import org.kuehntopf.streamtastic.core.Station;
import org.kuehntopf.streamtastic.utils.SelectionMaintainer;

/**
 * TableModel for displaying shoutcast stations
 * @author akuehntopf
 */
public class StationTableModel extends AbstractTableModel        
{     
    private Vector<Station> _stations = new Vector<Station>();
    private SelectionMaintainer<Station> selectionMaintainer = new SelectionMaintainer<Station>() {
        @Override
        public void fireReplaced(int start, int end)
        {
            fireTableRowsUpdated(start, end);
        }
        @Override
        public void fireRemoved(int start, int end)
        {
            fireTableRowsDeleted(start, end);
        }
        @Override
        public void fireAdded(int start, int end)
        {
            fireTableRowsInserted(start, end);
        }
    };
    
    public int getRowCount() 
    {
        return _stations.size();
    }

    public int getColumnCount() 
    {
        return Station.Property.values().length;
    }

    public Object getValueAt(int row, int col) 
    {
        return _stations.get(row);
    }
    
    public void setStations(List<Station> stations, int selectedRow)
    {
        selectionMaintainer.updateList(_stations, stations, selectedRow);
    }
   
    public void clear()
    {
        _stations.clear();
        fireTableDataChanged();
    }
}
