/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.models;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.core.config.Config.Key;
import org.kuehntopf.streamtastic.core.streamripper.StreamripperCommand;
import org.kuehntopf.streamtastic.gui.components.CustomEditor;
import org.kuehntopf.streamtastic.gui.components.ExpandableOptionPane;
import org.kuehntopf.streamtastic.gui.components.ExpandableOptionPane.ParamType;
import org.kuehntopf.streamtastic.utils.Player;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class StreamtasticOptionModel extends OptionModel {

    private JScrollPane pane;
    private List<ExpandableOptionPane> optionPanes;
    private AdvancedPreferencesModel model;
    private Key[] configParams = new Key[] {
            Key.DESTINATION_DIR,
            Key.DIR_STATION_DOUBLE_CLICK_ACTION,
            Key.FAVS_STATION_DOUBLE_CLICK_ACTION,
            Key.PREF_SHOW_ADVANCED_TAB,
            Key.ICECAST_GENRE_BLACKLIST,
            Key.LOOK_AND_FEEL,
            Key.MINIMIZE_TO_TRAY,
            Key.PLAYER_CMD,
            Key.PLAYER_OPTS,
            Key.SHOUTCAST_MAX_LIST_LENGTH,
            Key.STREAMRIPPER_CMD
    };

    public StreamtasticOptionModel(AdvancedPreferencesModel advancedPreferencesModel)
    {
        this.model = advancedPreferencesModel;
    }

    private JScrollPane initDisplay()
    {
        // constraints
        GridBagConstraints optionPaneConstraints = new GridBagConstraints();
        optionPaneConstraints.fill = GridBagConstraints.HORIZONTAL;
        optionPaneConstraints.gridx = GridBagConstraints.REMAINDER;
        optionPaneConstraints.insets = new Insets(5,5,5,5);
        optionPaneConstraints.weightx = 1;
        optionPaneConstraints.anchor = GridBagConstraints.NORTH;

        GridBagConstraints spacerConstraints = (GridBagConstraints) optionPaneConstraints.clone();
        spacerConstraints.fill = GridBagConstraints.BOTH;
        spacerConstraints.weighty = 1;

        optionPanes = new ArrayList<ExpandableOptionPane>(4);
        ButtonGroup buttonGroup = new ButtonGroup() {
            // modifies default behaviour, so that it is possible to
            // deselect/close the current selected ExpandleOptionPane
            @Override
            public void setSelected(ButtonModel m, boolean b)
            {
                if (!b && m!= null && m.isSelected()) clearSelection();
                else super.setSelected(m, b);
            }
        
        };

        ExpandableOptionPane guiOptions = new ExpandableOptionPane(Utilities.getString("PREF_TAB_ST_EXPANE_GUI_SETTINGS_TITLE"), false, buttonGroup);
        guiOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_LAF_OPTIONS_TITLE"));
        guiOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_LOOK_AND_FEEL_LABEL"), Config.Key.LOOK_AND_FEEL, ParamType.COMBO);
        {
            LookAndFeelInfo[] installedLafs = UIManager.getInstalledLookAndFeels();
            String[] labels = new String[installedLafs.length];
            Object[] values = new String[installedLafs.length];
            for (int i=0; i < installedLafs.length; i++)
            {
                labels[i] = installedLafs[i].getName();
                values[i] = installedLafs[i].getClassName();
            }
            guiOptions.setValueSetForCombo(Config.Key.LOOK_AND_FEEL, labels, values);
            //TODO custom laf (not installed, set from advanced tab) needs to be handled correct!
        }
        guiOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_DBL_CLICK_OPTIONS_TITLE"));
        guiOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_DIR_STATION_DOUBLE_CLICK_ACTION_LABEL"), Config.Key.DIR_STATION_DOUBLE_CLICK_ACTION, ParamType.COMBO, 1);
        {
            guiOptions.setValueSetForCombo(Config.Key.DIR_STATION_DOUBLE_CLICK_ACTION,
                    new String[] { Utilities.getString("PLAY"), Utilities.getString("RECORD") },
                    Config.Key.DIR_STATION_DOUBLE_CLICK_ACTION.getParameterInfo().getFixedValues());
        }
        guiOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_FAVS_STATION_DOUBLE_CLICK_ACTION_LABEL"), Config.Key.FAVS_STATION_DOUBLE_CLICK_ACTION, ParamType.COMBO, 1);
        {
            guiOptions.setValueSetForCombo(Config.Key.FAVS_STATION_DOUBLE_CLICK_ACTION,
                    new String[] { Utilities.getString("PLAY"), Utilities.getString("RECORD") },
                    Config.Key.FAVS_STATION_DOUBLE_CLICK_ACTION.getParameterInfo().getFixedValues());
        }
        guiOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_TRAY_OPTIONS_TITLE"));
        guiOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_MINIMIZE_TO_TRAY_LABEL"), Config.Key.MINIMIZE_TO_TRAY, ParamType.CB_SIMPLE);
        guiOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_ADVANCED_PREF_OPTIONS_TITLE"));
        guiOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_PREF_SHOW_ADVANCED_TAB_LABEL"), Config.Key.PREF_SHOW_ADVANCED_TAB, ParamType.CB_SIMPLE);
        optionPanes.add(guiOptions);

        ExpandableOptionPane srOptions = new ExpandableOptionPane(Utilities.getString("PREF_TAB_ST_EXPANE_STREAMRIPPER_SETTINGS_TITLE"), false, buttonGroup);
        srOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_SR_DEST_DIR_OPTIONS_TITLE"));
        srOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_STREAMRIPPER_CMD_LABEL"), Config.Key.STREAMRIPPER_CMD, ParamType.LONG_STRING);
        {
            srOptions.setCustomEditorForTextField(Config.Key.STREAMRIPPER_CMD, new CustomEditor() {
                public String edit(JComponent parent, String currentValue)
                {
                    File selectedFile = StreamripperCommand.showSelectStreamripperDialog(parent);
                    return (selectedFile==null)?currentValue:selectedFile.getAbsolutePath();
                }
            });
        }
        srOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_DESTINATION_DIR_LABEL"), Config.Key.DESTINATION_DIR, ParamType.LONG_STRING);
        {
            srOptions.setCustomEditorForTextField(Config.Key.DESTINATION_DIR, new CustomEditor() {
                public String edit(JComponent parent, String currentValue)
                {
                    File destDir = Utilities.showFileSelectionDialog(parent, Utilities
                            .getString("SELDEST_TITLE"), Utilities.getString("SELDEST_APPROVE_LABEL"),
                            currentValue, true);
                    return (destDir==null)? currentValue: destDir.getAbsolutePath();
                }
            });
        }
        optionPanes.add(srOptions);

        ExpandableOptionPane playerOptions = new ExpandableOptionPane(Utilities.getString("PREF_TAB_ST_EXPANE_PLAYER_SETTINGS_TITLE"), false, buttonGroup);
        playerOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_EXT_PLAYBACK_OPTIONS_TITLE"));
        playerOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_PLAYER_CMD_LABEL"), Config.Key.PLAYER_CMD, ParamType.LONG_STRING);
        {
            playerOptions.setCustomEditorForTextField(Config.Key.PLAYER_CMD, new CustomEditor() {
                public String edit(JComponent parent, String currentValue)
                {
                    File player = Player.showPlayerSelectionDialog(parent);
                    return (player == null)? currentValue: player.getAbsolutePath();
                }
            });
        }
        playerOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_PLAYER_OPTS_LABEL"), Config.Key.PLAYER_OPTS, ParamType.CB_LONG_STRING);
        optionPanes.add(playerOptions);

        ExpandableOptionPane directoryOptions = new ExpandableOptionPane(Utilities.getString("PREF_TAB_ST_EXPANE_STREAM_DIR_SETTINGS_TITLE"), false, buttonGroup);
        directoryOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_SHOUTCAST_OPTIONS_TITLE"));
        directoryOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_SHOUTCAST_MAX_LIST_LENGTH_LABEL"), Config.Key.SHOUTCAST_MAX_LIST_LENGTH, ParamType.CB_SPINNER);
        {
            directoryOptions.setModelForSpinner(Config.Key.SHOUTCAST_MAX_LIST_LENGTH, new SpinnerNumberModel(500,1,Integer.MAX_VALUE,1));
        }
        directoryOptions.startGroup(Utilities.getString("PREF_TAB_ST_GROUP_ICECAST_OPTIONS_TITLE"));
        directoryOptions.addParam(Utilities.getString("PREF_TAB_ST_PARAM_ICECAST_GENRE_BLACKLIST_LABEL"), Config.Key.ICECAST_GENRE_BLACKLIST, ParamType.TEXT_AREA);
        optionPanes.add(directoryOptions);

        buttonGroup.clearSelection();

        JPanel main = new JPanel();
        main.setLayout(new GridBagLayout());
        for (ExpandableOptionPane pane : optionPanes) main.add(pane, optionPaneConstraints);
        //add spacer
        main.add(Box.createVerticalGlue(),spacerConstraints);

        JScrollPane display = new JScrollPane(main);
        display.getVerticalScrollBar().setUnitIncrement(20);
        return display;
    }

    @Override
    public void readFromBaseModel()
    {
        for (ExpandableOptionPane pane : optionPanes)
            for (Enum param : pane.getParams())
                if (param == Key.SHOUTCAST_MAX_LIST_LENGTH
                        && model.getValue((Key) param)
                                .equals(Key.SHOUTCAST_MAX_LIST_LENGTH.getParameterInfo()
                                                .getFixedValues()[0]))
                    pane.setValue(param, null);
                else if (param == Key.LOOK_AND_FEEL)
                    pane.setValue(param, Config.getInstance().getLookAndFeelClassName(
                            model.getValue((Key) param)));
                else if (param == Key.MINIMIZE_TO_TRAY || param == Key.PREF_SHOW_ADVANCED_TAB)
                    pane.setValue(param, model.getValue((Key) param).equals(
                            ((Key) param).getParameterInfo().getFixedValues()[1]) ? ""
                            : null);
                else if (param == Key.PLAYER_OPTS
                        && Config.getInstance().getDefaultValue(Key.PLAYER_OPTS).equals(
                                model.getValue(Key.PLAYER_OPTS)))
                {
                    pane.setValue(param, model.getValue((Key) param));
                    pane.setValue(param, null);
                }
                else if (param instanceof Key) pane.setValue(param, model.getValue((Key) param));
    }

    @Override
    public void writeToBaseModel()
    {
        for (ExpandableOptionPane pane : optionPanes)
            for (Enum param : pane.getParams())
                if (param == Key.MINIMIZE_TO_TRAY || param == Key.PREF_SHOW_ADVANCED_TAB)
                    model.setValue((Key) param, pane.isSet(param) ? ((Key) param)
                            .getParameterInfo().getFixedValues()[1] : ((Key) param)
                            .getParameterInfo().getFixedValues()[0]);
                else if (param == Key.SHOUTCAST_MAX_LIST_LENGTH
                        && !pane.isSet(Key.SHOUTCAST_MAX_LIST_LENGTH))
                    model.setValue((Key) param, Key.SHOUTCAST_MAX_LIST_LENGTH.getParameterInfo()
                            .getFixedValues()[0]);
                else if (param == Key.PLAYER_OPTS && !pane.isSet(Key.PLAYER_OPTS)) 
                    model.restore(Key.PLAYER_OPTS);
                else if (param instanceof Key) model.setValue((Key) param, pane.getValue(param));
    }

    @Override
    public Container getDisplay()
    {
        if (pane==null) pane = initDisplay();
        return pane;
    }

    @Override
    public void applyChanges()
    {
        writeToBaseModel();
        for (Key param : configParams) model.apply(param);
        readFromBaseModel();
    }

    @Override
    public void loadDefaults()
    {
        for (Key param : configParams) model.restore(param);
        readFromBaseModel();
    }

    @Override
    public void revertChanges()
    {
        for (Key param : configParams) model.revert(param);
        readFromBaseModel();
    }
}
