/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.gui.views;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.kuehntopf.streamtastic.core.config.Config;
import org.kuehntopf.streamtastic.gui.StreamtasticFrame;
import org.kuehntopf.streamtastic.gui.models.AdvancedPreferencesModel;
import org.kuehntopf.streamtastic.gui.models.OptionModel;
import org.kuehntopf.streamtastic.gui.models.StreamRipperOptionModel;
import org.kuehntopf.streamtastic.gui.models.StreamtasticOptionModel;
import org.kuehntopf.streamtastic.utils.Utilities;

/**
 * @author Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 *
 */
public class PreferencesView extends ToolbarView {

    private StreamtasticFrame sframe;
    private Map<PrefAction, Action> actions;
    private AdvancedPreferencesModel advancedPreferencesModel;
    private OptionModel streamRipperOptionModel, streamtasticOptionModel;
    private JTabbedPane preferencesPane;
    private enum PrefAction {
        APPLY_ALL, REVERT_ALL, LOAD_DEFAULT_ALL
    }

    /**
     * @param title
     * @param headerIcon
     * @param displayIcon
     */
    public PreferencesView(StreamtasticFrame sframe)
    {
        super(Utilities.getString("SIDEBARMENU_PREFERENCES"), Utilities
                .loadIcon("large/preferences.png"), Utilities.loadIcon("small/preferences.png"));
        this.sframe = sframe;
        initActions();
        init();
    }

    private void initActions()
    {
        actions = new TreeMap<PrefAction, Action>();
        actions.put(PrefAction.APPLY_ALL, new AbstractAction(Utilities
                .getString("PREF_ACTION_APPLY_ALL"), Utilities.loadIcon("small/accept.png")) {
            {
                this.putValue(Action.SHORT_DESCRIPTION, Utilities.getString("PREF_ACTION_APPLY_ALL"));
            }
            //TODO disable action if there is nothing to apply
            public void actionPerformed(ActionEvent e)
            {
                // streamripper tab
                if (preferencesPane.getSelectedComponent() == streamRipperOptionModel.getDisplay())
                    streamRipperOptionModel.applyChanges();

                // streamtastic tab
                else if (preferencesPane.getSelectedComponent() == streamtasticOptionModel.getDisplay())
                    streamtasticOptionModel.applyChanges();

                else advancedPreferencesModel.applyAll();
            }
        
        });
        actions.put(PrefAction.REVERT_ALL, new AbstractAction(Utilities
                .getString("PREF_ACTION_REVERT_ALL"), Utilities.loadIcon("small/cancel.png")) {
            {
                this.putValue(Action.SHORT_DESCRIPTION, Utilities.getString("PREF_ACTION_REVERT_ALL"));
            }
            //TODO disable action if there is nothing to discard
            public void actionPerformed(ActionEvent e)
            {
                // streamripper tab
                if (preferencesPane.getSelectedComponent() == streamRipperOptionModel.getDisplay())
                    streamRipperOptionModel.revertChanges();

                // streamtastic tab
                else if (preferencesPane.getSelectedComponent() == streamtasticOptionModel.getDisplay())
                    streamtasticOptionModel.revertChanges();

                else advancedPreferencesModel.revertAll();
            }
        
        });
        actions.put(PrefAction.LOAD_DEFAULT_ALL, new AbstractAction(Utilities
                .getString("PREF_ACTION_RESTORE_ALL"), Utilities.loadIcon("small/cancel_all.png")) {
            {
                this.putValue(Action.SHORT_DESCRIPTION, Utilities.getString("PREF_ACTION_RESTORE_ALL"));
            }
            //TODO disable action if there is nothing to restore
            public void actionPerformed(ActionEvent e)
            {
                // ask whether default values should be loaded
                if (JOptionPane.NO_OPTION == JOptionPane.showConfirmDialog(sframe, Utilities
                        .getString("PREF_ACTION_RESTORE_ALL_DLG_WARNING_MSG"), Utilities
                        .getString("PREF_ACTION_RESTORE_ALL_DLG_WARNING_TITLE"),
                        JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE))
                    return;

                // streamripper tab
                if (preferencesPane.getSelectedComponent() == streamRipperOptionModel.getDisplay())
                    streamRipperOptionModel.loadDefaults();

                // streamtastic tab
                else if (preferencesPane.getSelectedComponent() == streamtasticOptionModel.getDisplay())
                    streamtasticOptionModel.loadDefaults();

                else advancedPreferencesModel.restoreAll();
            }
        
        });
    }

    /* (non-Javadoc)
     * @see org.kuehntopf.streamtastic.gui.views.ToolbarView#initToolbar(javax.swing.JToolBar)
     */
    @Override
    protected void initToolbar(JToolBar toolbar)
    {
        toolbar.add(actions.get(PrefAction.APPLY_ALL));
        toolbar.add(actions.get(PrefAction.REVERT_ALL));
        toolbar.add(actions.get(PrefAction.LOAD_DEFAULT_ALL));
    }

    /* (non-Javadoc)
     * @see org.kuehntopf.streamtastic.gui.views.View#activate()
     */
    @Override
    protected void activate()
    {
        streamtasticOptionModel.readFromBaseModel();
        streamRipperOptionModel.readFromBaseModel();
    }

    @Override
    public boolean isReadyForViewChange()
    {
        // make sure current changes are writen to base model
        streamtasticOptionModel.writeToBaseModel();
        streamRipperOptionModel.writeToBaseModel();

        boolean veto = false;
        if (advancedPreferencesModel.isModified())
            switch (JOptionPane.showOptionDialog(this, Utilities
                    .getString("PREF_DLG_UNAPPLIED_CHANGES_MSG"), Utilities
                    .getString("PREF_DLG_UNAPPLIED_CHANGES_TITLE"), JOptionPane.DEFAULT_OPTION,
                    JOptionPane.WARNING_MESSAGE, null, new Object[] {
                            Utilities.getString("PREF_DLG_UNAPPLIED_CHANGES_OPT_APPLY_ALL"),
                            Utilities.getString("PREF_DLG_UNAPPLIED_CHANGES_OPT_REVERT_ALL"),
                            Utilities.getString("PREF_DLG_UNAPPLIED_CHANGES_OPT_CANCEL") },
                    Utilities.getString("PREF_DLG_UNAPPLIED_CHANGES_OPT_CANCEL")))
            {
                case 0: //apply
                    streamtasticOptionModel.applyChanges();
                    streamRipperOptionModel.applyChanges();
                    break;
                case 1: //revert
                    streamtasticOptionModel.revertChanges();
                    streamRipperOptionModel.revertChanges();
                    break;
                default: //cancel
                    veto = true;
            }
        return !veto;
    }

    /* (non-Javadoc)
     * @see org.kuehntopf.streamtastic.gui.views.View#initContentPane()
     */
    @Override
    protected JComponent initContentPane()
    {
        preferencesPane = new JTabbedPane();
        advancedPreferencesModel = new AdvancedPreferencesModel();

        //TODO on tab change check if there are changes left to apply, same if view is left
        streamtasticOptionModel = new StreamtasticOptionModel(advancedPreferencesModel);
        preferencesPane.add(Utilities.getString("PREF_TAB_ST_TITLE"), streamtasticOptionModel.getDisplay());

        streamRipperOptionModel = new StreamRipperOptionModel(advancedPreferencesModel);
        preferencesPane.add(Utilities.getString("PREF_TAB_SR_TITLE"), streamRipperOptionModel.getDisplay());

        final JTable advancedConfig = new JTable(advancedPreferencesModel, advancedPreferencesModel.getColumnModel());
        // 4 extra pixels for the cell editors
        advancedConfig.setRowHeight(advancedConfig.getRowHeight()+4);
        if (Config.getInstance().getProperty(Config.Key.PREF_SHOW_ADVANCED_TAB).equals(
                Config.Key.PREF_SHOW_ADVANCED_TAB.getParameterInfo().getFixedValues()[1]))
            preferencesPane.add(Utilities.getString("PREF_TAB_ADVANCED_TITLE"), new JScrollPane(
                    advancedConfig));
        Config.getInstance().addPropertyChangeListener(Config.Key.PREF_SHOW_ADVANCED_TAB,
            new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt)
                {
                    if (Config.getInstance().getProperty(Config.Key.PREF_SHOW_ADVANCED_TAB)
                                .equals(Config.Key.PREF_SHOW_ADVANCED_TAB.getParameterInfo()
                                                .getFixedValues()[1]))
                    {
                        preferencesPane.add(Utilities.getString("PREF_TAB_ADVANCED_TITLE"),
                                new JScrollPane(advancedConfig));
                    }
                    else
                    {
                        preferencesPane.removeTabAt(preferencesPane.indexOfTab(Utilities
                                .getString("PREF_TAB_ADVANCED_TITLE")));
                    }
                }
            });

        preferencesPane.setBorder(new CompoundBorder(new EmptyBorder(4,0,0,0), preferencesPane.getBorder()));
        return preferencesPane;
    }
}