/*
 * Streamtastic Streamripper Frontend
 * Copyright 2008 Andreas Kuehntopf <andreas@kuehntopf.org>
 * Copyright 2008-2010 Michael Lippold <thabird@free.fr>
 * Copyright 2008-2011 Joachim Lippold <mailToJoeAtIcq@icqmail.com>
 * 
 * This file is part of Streamtastic.
 *
 * Streamtastic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Streamtastic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Streamtastic.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.kuehntopf.streamtastic.utils;

import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.kuehntopf.streamtastic.core.config.Config;

/**
 * Player class
 * Provides easy access to an external player
 * @author akuehntopf
 */
public class Player 
{
    private static String url = null;
    public static final String OPTION_URL = "url";
    private static Command playerCmd = null;
    static
    {
        try {
            String player = Config.getInstance().getProperty(Config.Key.PLAYER_CMD);
            if (player!=null) playerCmd = new Command(player, Config
                    .getInstance().getProperty(Config.Key.PLAYER_OPTS));
        } catch (Exception e) { e.printStackTrace(); }

        PropertyChangeListener playerPropsChangeListener = new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt)
            {
                if (evt.getPropertyName().equals(Config.Key.PLAYER_CMD.toString()))
                    try {
                        playerCmd = null;
                        String player = Config.getInstance().getProperty(Config.Key.PLAYER_CMD);
                        if (player != null)
                            playerCmd = new Command(player, Config.getInstance().getProperty(
                                    Config.Key.PLAYER_OPTS));
                    } catch (Exception e) { e.printStackTrace(); }
                else if (playerCmd != null)
                    playerCmd.setOptionString(Config.getInstance().getProperty(Config.Key.PLAYER_OPTS));
            }
        };
        Config.getInstance().addPropertyChangeListener(Config.Key.PLAYER_CMD, playerPropsChangeListener);
        Config.getInstance().addPropertyChangeListener(Config.Key.PLAYER_OPTS, playerPropsChangeListener);
    }
    
    /**
     * Starts the player
     * with the specified URL as argument
     */
    public static void
    play(String url) 
    throws UnsupportedOperationException
    {
        // Check if player is available.
        // throw an exception if it isn't
        if (!Config.getInstance().isValueValidForKey(Config.Key.PLAYER_CMD) || playerCmd == null)
            throw new UnsupportedOperationException("Player not set");

        // Launch the Player
        try
        {
            playerCmd.set(OPTION_URL, url);
            playerCmd.execute();
            Player.url = url;
        }
        catch (IOException e)
        {
            Player.url = null;
            throw new UnsupportedOperationException("Error invoking player");
        }
    }
    
    public static boolean
    isPlaying(String url)
    {
        if (url==null) 
            return false;
        return url.equals(Player.url);
    }
    
    public static File
    showPlayerSelectionDialog(Component parent) 
    {
        String location = Config.getInstance().getProperty(Config.Key.PLAYER_CMD);
        if (location == null) location = Utilities.getHome();
        
        JFileChooser filechooser = new JFileChooser(location);
        filechooser.setAcceptAllFileFilterUsed(false);
        filechooser.setDialogTitle(Utilities.getString("PLAY_SELECT_PLAYER_TITLE"));
        filechooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        filechooser.setFileFilter(new FileFilter() {
            boolean isWin32 = System.getProperty("os.name").contains("Windows");
            @Override
            public String getDescription()
            {
                return Utilities.getString("PLAY_SELECT_PLAYER_FILETYPE");
            }
            @Override
            public boolean accept(File f)
            {
                return f.isDirectory() || canExecute(f);
            }
            private boolean canExecute(File f) 
            {
                if (isWin32) return f.getName().matches(".*\\.(exe|com|bat)");
                return f.canExecute();
            }
        });
        if (JFileChooser.APPROVE_OPTION == filechooser.showDialog(parent,
                Utilities.getString("PLAY_SELECT_PLAYER_BUTTON_TEXT")))
            return filechooser.getSelectedFile();
        
        return null;
    }

}
